package com.ruoyi.demo.service.impl;

import cn.hutool.core.lang.Dict;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.ruoyi.demo.entity.*;
import com.ruoyi.demo.entity.bo.PlanAddBo;
import com.ruoyi.demo.entity.bo.PlanBo;
import com.ruoyi.demo.entity.bo.PlanItemAddBo;
import com.ruoyi.demo.entity.vo.*;
import com.ruoyi.demo.mapper.*;
import com.ruoyi.demo.service.PlanService;
import com.ruoyi.demo.utils.SignUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

@Service
@Transactional
public class PlanServiceImpl implements PlanService {
    @Autowired
    PlanMapper planMapper;
    @Autowired
    PlanItemMapper planItemMapper;
    @Autowired
    PlanAuditMapper planAuditMapper;
    @Autowired
    AppMapper appMapper;
    @Autowired
    CommercialMapper commercialMapper;
    @Autowired
    SystemSetterMapper systemSetterMapper;
    @Autowired
    StoreMapper storeMapper;
    @Autowired
    PlanPushStatisticsMapper planPushStatisticsMapper;
    @Autowired
    AddrCategoryMapper addrCategoryMapper;
    @Autowired
    EquipmentMapper equipmentMapper;
    @Autowired
    GoodsMapper goodsMapper;

    @Override
    public int addPlan(PlanAddBo bo) {
        QueryWrapper<App> appQueryWrapper = new QueryWrapper<>();
        appQueryWrapper.eq("appkey",bo.getAppkey());
        appQueryWrapper.eq("deleted","0");
        App app = appMapper.selectOne(appQueryWrapper);
        if (app == null)
            return -1;
        Map<String,Object> params = Dict.create()
            .set("appkey",bo.getAppkey())
            .set("goodsId",bo.getGoodsId())
            .set("goodsImg",bo.getGoodsImg())
            .set("goodsName",bo.getGoodsName())
            .set("goodsLogo",bo.getGoodsLogo())
            .set("goodsIntroduce",bo.getGoodsIntroduce())
            .set("goodsPrice",bo.getGoodsPrice())
            .set("goodsClassify",bo.getGoodsClassify())
            .set("planStartTime",bo.getPlanStartTime())
            .set("planEndTime",bo.getPlanEndTime())
            .set("items",bo.getItems())
            .set("sign",bo.getSign());
        if (!SignUtil.verify(params,app.getAppSecret()))
            return -3;
        // 插入广告计划
        Plan plan = new Plan(app,bo);
        planMapper.insert(plan);
        // 插入广告计划明细
        for (PlanItemAddBo itemAddBo : bo.getItems()) {
            PlanItem item = new PlanItem(plan.getPlanId(), itemAddBo);
            planItemMapper.insert(item);
        }
        // 插入商品信息
        QueryWrapper<Goods> goodsQueryWrapper = new QueryWrapper<>();
        goodsQueryWrapper.eq("appkey",bo.getAppkey());
        goodsQueryWrapper.eq("goods_id",bo.getGoodsId());
        Goods query = goodsMapper.selectOne(goodsQueryWrapper);
        if (query == null){
            Goods goods = new Goods(plan);
            goodsMapper.insert(goods);
        }
        return 1;
    }

    @Override
    public int deletePlan(String id) {
        QueryWrapper<PlanItem> itemQueryWrapper = new QueryWrapper<>();
        itemQueryWrapper.eq("plan_id",id);
        planItemMapper.delete(itemQueryWrapper);
        QueryWrapper<Plan> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("plan_id",id);
        return planMapper.delete(queryWrapper);
    }

    @Override
    public int deletePlans(List<String> ids) {
        int i = 0;
        for (String id : ids){
            QueryWrapper<Plan> queryWrapper = new QueryWrapper<>();
            queryWrapper.eq("plan_id",id);
            i += planMapper.delete(queryWrapper);
        }
        return i;
    }

    @Override
    public int checkPlan(PlanBo bo,String userId) {
        QueryWrapper<Plan> planQueryWrapper = new QueryWrapper<>();
        planQueryWrapper.eq("plan_id",bo.getPlanId());
        planQueryWrapper.eq("deleted","0");
        Plan plan = planMapper.selectOne(planQueryWrapper);
        if (plan == null)
            return -1;
        plan.setState(bo.getState());
        int i = planMapper.updateById(plan);
        if (i == 0)
            return 0;
        PlanAudit audit = new PlanAudit(bo,userId);
        planAuditMapper.insert(audit);
        return 1;
    }

    @Override
    public PageInfo<List<PlanListVo>> getPlanList(PlanBo bo) {
        PageHelper.startPage(bo.getPageNum(), bo.getPageSize(),true);
        QueryWrapper<Plan> planQueryWrapper = new QueryWrapper<>();
        if (bo.getText() != null){
            planQueryWrapper.like("plan_id",bo.getText())
                .or()
                .like("goods_name",bo.getText());
        }
        planQueryWrapper.eq("deleted","0");
        List<Plan> plans = planMapper.selectList(planQueryWrapper);
        List<PlanListVo> vos = new ArrayList<>();
        for (Plan plan : plans){
            vos.add(new PlanListVo(plan));
        }
        return new PageInfo(vos);
    }

    @Override
    public PlanDetailVo getPlanDetail(String planId) {
        QueryWrapper<Plan> planQueryWrapper = new QueryWrapper<>();
        planQueryWrapper.eq("plan_id",planId);
        planQueryWrapper.eq("deleted","0");
        Plan plan = planMapper.selectOne(planQueryWrapper);
        if (plan == null)
            return null;
        QueryWrapper<PlanItem> itemQueryWrapper = new QueryWrapper<>();
        itemQueryWrapper.eq("plan_id",planId);
        List<PlanItem> items = planItemMapper.selectList(itemQueryWrapper);
        List<PlanItemVo> itemVos = new ArrayList<>();
        if (items != null)
            for (PlanItem item : items){
                itemVos.add(new PlanItemVo(item));
            }
        PlanDetailVo vo = new PlanDetailVo(plan, itemVos);
        return vo;
    }

    @Override
    public List<PlanPushVo> getPlanPush() {
//        查询可投放的广告计划
//        再根据每个广告计划，查询其明细
//        返回结果
        QueryWrapper<SystemSetter> systemSetterQueryWrapper = new QueryWrapper<>();
        SystemSetter setter = systemSetterMapper.selectOne(systemSetterQueryWrapper);
        QueryWrapper<Plan> planQueryWrapper = new QueryWrapper<>();
        planQueryWrapper.apply("DATE_FORMAT(plan_start_time,'%Y-%m-%d %H:%i:%s') <= DATE_FORMAT({0},'%Y-%m-%d %H:%i:%s')",LocalDateTime.now());
        planQueryWrapper.apply("DATE_FORMAT(plan_end_time,'%Y-%m-%d %H:%i:%s') > DATE_FORMAT({0},'%Y-%m-%d %H:%i:%s')",LocalDateTime.now());
        planQueryWrapper.eq("state",1);
        planQueryWrapper.eq("deleted","0");
        List<PlanPushVo> pushVos = new ArrayList<>();
        for (Plan plan : planMapper.selectList(planQueryWrapper)) {
            QueryWrapper<PlanItem> itemQueryWrapper = new QueryWrapper<>();
            itemQueryWrapper.eq("plan_id",plan.getPlanId());
            for (PlanItem item : planItemMapper.selectList(itemQueryWrapper)) {
                pushVos.add(new PlanPushVo(item,plan,setter));
            }
        }
        return pushVos;
    }

    @Override
    public int planUpDown(String planId) {
        QueryWrapper<Plan> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("plan_id",planId);
        queryWrapper.eq("deleted","0");
        Plan plan = planMapper.selectOne(queryWrapper);
        if (plan == null)
            return -1;
        plan.setOnState(plan.getOnState()+1);
        if (plan.getOnState() >= 2)
            plan.setOnState(0);
        return planMapper.updateById(plan);
    }

    @Override
    public PageInfo<List<PlanManageListVo>> getPlanManageList(PlanBo bo) {
        PageHelper.startPage(bo.getPageNum(),bo.getPageSize(),true);
        QueryWrapper<Plan> planQueryWrapper = new QueryWrapper<>();
        planQueryWrapper.eq("state",bo.getState());
        if (bo.getText() != null)
            planQueryWrapper.like("plan_id",bo.getText())
                .or()
                .like("principal_name",bo.getText())
                .or()
                .like("principal_telephone",bo.getText());
        if (bo.getStartTime() != null)
            planQueryWrapper.apply("DATE_FORMAT(plan_start_time,'%Y-%m-%d %H:%i:%s') <= DATE_FORMAT({0},'%Y-%m-%d %H:%i:%s')",bo.getStartTime());
        if (bo.getEndTime() != null)
            planQueryWrapper.apply("DATE_FORMAT(plan_end_time,'%Y-%m-%d %H:%i:%s') > DATE_FORMAT({0},'%Y-%m-%d %H:%i:%s')",bo.getEndTime());
        planQueryWrapper.eq("deleted","0");
        List<Plan> plans = planMapper.selectList(planQueryWrapper);
        List<PlanManageListVo> vos = new ArrayList<>();
        for (Plan plan : plans){
            PlanManageListVo vo = new PlanManageListVo(plan);
            Integer count = planItemMapper.selectEquipmentCountByPlanId(plan.getPlanId());
            vo.setEquipmentCount(count);
            vos.add(vo);
        }
        return new PageInfo(vos);
    }

    @Override
    public PlanManageDetailVo getPlanManageDetail(String planId) {
        QueryWrapper<Plan> planQueryWrapper = new QueryWrapper<>();
        planQueryWrapper.eq("plan_id",planId);
        planQueryWrapper.eq("deleted","0");
        Plan plan = planMapper.selectOne(planQueryWrapper);
        if (plan == null)
            return null;
        QueryWrapper<PlanItem> itemQueryWrapper = new QueryWrapper<>();
        itemQueryWrapper.eq("plan_id",planId);
        itemQueryWrapper.eq("deleted","0");
        List<PlanItem> items = planItemMapper.selectList(itemQueryWrapper);
        List<PlanManageItemVo> itemVos = new ArrayList<>();
        for (PlanItem item : items){
            PlanManageItemVo itemVo = new PlanManageItemVo(item);
            Store store = storeMapper.selectById(item.getStoreId());
            itemVo.setStoreName(store.getStoreName());
            itemVos.add(itemVo);
        }
        PlanManageDetailVo vo = new PlanManageDetailVo(plan, itemVos);
        return vo;
    }

    @Override
    public PageInfo<List<PlanPushListVo>> getPlanPushList(PlanBo bo) {
        PageHelper.startPage(bo.getPageNum(),bo.getPageSize(),true);
        QueryWrapper<Plan> planQueryWrapper = new QueryWrapper<>();
        planQueryWrapper.eq("appkey",bo.getAppkey());
        planQueryWrapper.eq("commercial_id",bo.getCommercialId());
        if (bo.getText() != null)
            planQueryWrapper.like("plan_id",bo.getText())
                .or()
                .like("plan_name",bo.getText())
                .or()
                .like("principal_name",bo.getText())
                .or()
                .like("principal_telephone",bo.getText());
        if (bo.getStartTime() != null)
            planQueryWrapper.apply("DATE_FORMAT(plan_start_time,'%Y-%m-%d %H:%i:%s') <= DATE_FORMAT({0},'%Y-%m-%d %H:%i:%s')",bo.getStartTime());
        if (bo.getEndTime() != null)
            planQueryWrapper.apply("DATE_FORMAT(plan_end_time,'%Y-%m-%d %H:%i:%s') > DATE_FORMAT({0},'%Y-%m-%d %H:%i:%s')",bo.getEndTime());
        planQueryWrapper.eq("deleted","0");
        List<Plan> plans = planMapper.selectList(planQueryWrapper);
        List<PlanPushListVo> vos = new ArrayList<>();
        for (Plan plan : plans){
            PlanPushListVo vo = new PlanPushListVo(plan);
            Integer count = planItemMapper.selectEquipmentCountByPlanId(plan.getPlanId());
            vo.setEquipmentCount(count);
            vos.add(vo);
        }
        return new PageInfo(vos);
    }

    @Override
    public PageInfo<List<PlanPushStatisticsListVo>> getPlanPushStatisticsList(PlanBo bo) {
        PageHelper.startPage(bo.getPageNum(),bo.getPageSize(),true);
        QueryWrapper<PlanPushStatistics> statisticsQueryWrapper = new QueryWrapper<>();
        if (bo.getText() != null)
            statisticsQueryWrapper.like("plan_id",bo.getText())
                .or()
                .like("equipment_id",bo.getText())
                .or()
                .like("equipment_name",bo.getText());
        if (bo.getStartTime() != null)
            statisticsQueryWrapper.apply("DATE_FORMAT(laster_time,'%Y-%m-%d %H:%i:%s') >= DATE_FORMAT({0},'%Y-%m-%d %H:%i:%s')",bo.getStartTime());
        if (bo.getEndTime() != null)
            statisticsQueryWrapper.apply("DATE_FORMAT(laster_time,'%Y-%m-%d %H:%i:%s') <= DATE_FORMAT({0},'%Y-%m-%d %H:%i:%s')",bo.getEndTime());
        List<PlanPushStatistics> list = planPushStatisticsMapper.selectList(statisticsQueryWrapper);
        List<PlanPushStatisticsListVo> vos = new ArrayList<>();
        for (PlanPushStatistics statistics : list){
            vos.add(new PlanPushStatisticsListVo(statistics));
        }
        return new PageInfo(vos);
    }

    @Override
    public PlanHomeVo getPlanTotal() {
        PlanHomeVo vo = new PlanHomeVo();
        vo.setTotal(planMapper.selectPlanTotal());
        return vo;
    }

    @Override
    public List<AddrCategory> getAddrCategory() {
        return addrCategoryMapper.selectList(null);
    }

    @Override
    public PageInfo<List<Equipment>> getEquipments(PlanBo bo) {
        PageHelper.startPage(bo.getPageNum(),bo.getPageSize(),true);
        QueryWrapper<Equipment> equipmentQueryWrapper = new QueryWrapper<>();
        if (bo.getText() != null)
            equipmentQueryWrapper.like("equipment_id",bo.getText())
                .or()
                .like("equipment_name",bo.getText());
        if (bo.getAddrCodes() != null){
            for (String addrCode : bo.getAddrCodes()){
                String province = String.valueOf(addrCode.charAt(0))+String.valueOf(addrCode.charAt(1));
                String city = String.valueOf(addrCode.charAt(2))+String.valueOf(addrCode.charAt(3));
                String district = String.valueOf(addrCode.charAt(4))+String.valueOf(addrCode.charAt(5));
                if (district.equals("00")){
                    if (city.equals("00"))
                        equipmentQueryWrapper.or().likeRight("addr_code",province);
                    else
                        equipmentQueryWrapper.or().likeRight("addr_code",province+city);
                }else
                    equipmentQueryWrapper.or().likeRight("addr_code",province+city+district);
            }
        }
        return new PageInfo(equipmentMapper.selectList(equipmentQueryWrapper));
    }
}
